/*
 Copyright (c) 2002 Amduus Information Works, Inc.  
 scott_auge@yahoo.com sauge@amduus.com

 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation the
 rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 sell copies of the Software, and to permit persons to whom the Software
 is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
 KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
*/ 

/* Take an algebraic expression and convert into RPN */
/* Needs to be included with CalcRPN.i               */

FUNCTION CalcAlg RETURNS CHARACTER (INPUT cExpression AS CHARACTER) FORWARD.

/* Locate first multiplication in expression, calc RPN, and replace */
/* the expression with the result.                                  */

FUNCTION SubMultiply RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR iWhereMult AS INTEGER NO-UNDO.
  DEF VAR i          AS INTEGER NO-UNDO.
  DEF VAR cMultExpr  AS CHARACTER NO-UNDO.
  DEF VAR cOpr1      AS CHARACTER NO-UNDO.
  DEF VAR cOpr2     AS CHARACTER NO-UNDO.

  /* IF there is no multiplier, then bail */

  IF INDEX (cExpression, "*") = 0 THEN RETURN cExpression.

  /* Locate a multiplier operator and it's operands               */
  /* This is a bad loop, should use index, but convenient to work */
  /* with ENTRY than INDEX.                                       */

  DO i = 1 TO NUM-ENTRIES (cExpression, " "):

     IF ENTRY(i, cExpression, " ") = "*" THEN DO:
       ASSIGN iWhereMult = i.
       LEAVE.
     END. /* IF ENTRY(i, cExpression, " ") = "*" */
       
  END. /* DO i = 1 TO NUM-ENTRIES (cExpression, " ") */

  /* Pull out our multiplication expression for substitution */

  ASSIGN
    cOpr1 = ENTRY(iWhereMult - 1, cExpression, " ")
    cOpr2 = ENTRY(iWhereMult + 1, cExpression, " ").

  /* Call out to RPNCalc to get a result to replace expression with */

  ASSIGN cMultExpr = STRING(CalcRPN(cOpr1 + " " + cOpr2 + " *")).

  /* Perform the substitution in the expression with the */
  /* calculated result.                                  */

  ASSIGN cExpression = REPLACE (cExpression, cOpr1 + " * " + cOpr2, cMultExpr).

  RETURN cExpression.

END. /* FUNCTION SubMultiply */

/* Locate first division in expression, calc RPN, and replace */
/* the expression with the result.                            */

FUNCTION SubDivision RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR iWhereDiv AS INTEGER NO-UNDO.
  DEF VAR i          AS INTEGER NO-UNDO.
  DEF VAR cDivExpr  AS CHARACTER NO-UNDO.
  DEF VAR cOpr1      AS CHARACTER NO-UNDO.
  DEF VAR cOpr2     AS CHARACTER NO-UNDO.

  /* IF there is no divisor, then bail */

  IF INDEX (cExpression, "/") = 0 THEN RETURN cExpression.

  /* Locate a divisor operator and it's operands                  */
  /* This is a bad loop, should use index, but convenient to work */
  /* with ENTRY than INDEX.                                       */

  DO i = 1 TO NUM-ENTRIES (cExpression, " "):

     IF ENTRY(i, cExpression, " ") = "/" THEN DO:
       ASSIGN iWhereDiv = i.
       LEAVE.
     END. /* IF ENTRY(i, cExpression, " ") = "/" */
       
  END. /* DO i = 1 TO NUM-ENTRIES (cExpression, " ") */

  /* Pull out our division expression for substitution */

  ASSIGN
    cOpr1 = ENTRY(iWhereDiv - 1, cExpression, " ")
    cOpr2 = ENTRY(iWhereDiv + 1, cExpression, " ").

  /* Call out to RPNCalc to get a result to replace expression with */

  ASSIGN cDivExpr = STRING(CalcRPN(cOpr1 + " " + cOpr2 + " /")).

  /* Perform the substitution in the expression with the */
  /* calculated result.                                  */

  ASSIGN cExpression = REPLACE (cExpression, cOpr1 + " / " + cOpr2, cDivExpr).

  RETURN cExpression.

END. /* FUNCTION SubDivision */

/* Function looks for groupings by () and evaluates the expression, */
/* replacing the () groupings with the evaluations.                 */
/* Note it looks for _( where _ is a space.  This way it won't mess */
/* with ( associated with functions which need the ( abutted to the */
/* function name as in "sin(" not "sin ("                           */
/* Hence, if you want groupings you must do this 3 + ( 9 - 6 ), NOT */
/* 3 +(9 - 6).                                                      */
/* Note, for functions, I am thinking of [ ] as argument delimiters */

FUNCTION GrpOrder RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR cSubExpression      AS CHARACTER NO-UNDO.
  DEF VAR iSubExpressionIndex AS INTEGER NO-UNDO.
  DEF VAR cEvalExpression     AS CHARACTER NO-UNDO.

  REPEAT:

    /* Nothing to group evaluate by. */

    IF INDEX(cExpression, " (") = 0 THEN RETURN TRIM(cExpression).
  
    /* Chop out part of the expression with _(_ in it       */
    /* to the next _)_ symbol.                              */
    /* We move right to left as the symbols tend to be      */
    /* present later in the string but need to be evaluated */
    /* before, if that makes any sense to the reader.       */
  
    ASSIGN cSubExpression = SUBSTRING(cExpression, R-INDEX(cExpression, " ( ")).
/* MESSAGE "OrderGrp:cSubExpression:1: " cSubExpression  . */
    /* Find the compliment _)_ to the _(_ */
  
    ASSIGN cSubExpression = SUBSTRING(cSubExpression, 1, INDEX(cSubExpression, " ) ")).
/* MESSAGE "OrderGrp:cSubExpression:2: " cSubExpression  . */
  
    /* Remove the _(_ and _)_ and send the sub-expression on it's */
    /* way to Alg2PRN for evaluation.                             */
  
    ASSIGN cSubExpression = REPLACE(cSubExpression, " ( ", "").
    ASSIGN cSubExpression = REPLACE(cSubExpression, " ) ", "").
    ASSIGN cSubExpression = TRIM(cSubExpression).
/* MESSAGE "OrderGrp:cSubExpression:3: " cSubExpression  . */
  
    ASSIGN cEvalExpression = CalcAlg(cSubExpression).
  
    /* Replace the sub-expression with the evaluated expression */

    ASSIGN cExpression = REPLACE(cExpression, "( " + cSubExpression + " )", cEvalExpression).

  END. /* REPEAT */

END. /* FUNCTION GrpOrder () */

/* Looks for a * or / in the expression and calls out to Subxxxx */
/* to perform the calculation and replacement.  Goes left to     */
/* right though the expression till nothing but + and - left.    */

FUNCTION OrderOpr RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  /* Determine if there are () grouping symbols and evaluate */
  /* accordingly.                                            */

  ASSIGN cExpression = GrpOrder(cExpression).

  /* Calculate functions here, they should reduce to numbers */
  /* for basic operators to work on.                         */

  ASSIGN cExpression = FuncEval (cExpression).

  REPEAT:

    /* Determine our stop condition */

    IF INDEX (cExpression, "*") = 0 AND INDEX (cExpression, "/") = 0 THEN RETURN cExpression.

    /* Determine which operation occurs first and call it's calculator */

    /* There is nothing but * in the expression */

    IF INDEX (cExpression, "/") = 0 THEN DO:
      ASSIGN cExpression =  SubMultiply(cExpression).
      NEXT.
    END.

    /* There is nothing but / in the expression */

    IF INDEX (cExpression, "*") = 0 THEN DO:
      ASSIGN cExpression =  SubDivision(cExpression).
      NEXT.
    END.

    /* We have * and / in the expression */

    IF INDEX (cExpression, "*") < INDEX (cExpression, "/") THEN 
        ASSIGN cExpression = SubMultiply(cExpression).
      ELSE 
        ASSIGN cExpression = SubDivision(cExpression).

  END. /* REPEAT */

END. /* FUNCTION OrderOpr */

FUNCTION Alg2RPN RETURNS CHARACTER (INPUT cExpression AS CHARACTER ):

  DEF VAR i            AS INTEGER NO-UNDO.
  DEF VAR cToken       AS CHARACTER NO-UNDO.
  DEF VAR cAccumulator AS CHARACTER NO-UNDO.
  DEF VAR cRegX        AS CHARACTER NO-UNDO.
  DEF VAR cRPN         AS CHARACTER NO-UNDO.

  /* Apply Order of Operations for Multiplication and division. */
  /* Next version will deal with () order of operations.         */

  ASSIGN cExpression = OrderOpr(cExpression).

  /* First Number in Algebra always Accumulator. */

  ASSIGN cRPN = ENTRY(1, cExpression, " ").

  DO i = 2 TO NUM-ENTRIES(cExpression, " "):

    ASSIGN cToken = ENTRY(i, cExpression, " ").

    IF IsOper(cToken) THEN DO:

      /* Remember the operator token */
      ASSIGN cRegX = cToken.

      ASSIGN i = i + 1.  /* Move to next operand */
      ASSIGN cToken = ENTRY(i, cExpression, " ").

      /* Attach Operand and operation into RPN string */
      ASSIGN cRPN = cRPN
                  + " "
                  + cToken
                  + " "
                  + cRegX. 
    END. /* IF IsOper(cToken) */

  END. /* DO */

  RETURN cRPN.

END. /* FUNCTION Alg2RPN */

/* Entry point for algebraic calculations */

FUNCTION CalcAlg RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  RETURN STRING(CalcRPN(Alg2RPN(CleanExpr(cExpression)))).

END.

