/*
 Copyright (c) 2002 Amduus Information Works, Inc.  
 scott_auge@yahoo.com sauge@amduus.com

 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation the
 rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 sell copies of the Software, and to permit persons to whom the Software
 is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
 KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
*/ 

FUNCTION IsOper RETURNS LOGICAL (INPUT cExpression AS CHARACTER):

  IF INDEX ("+-*/", cExpression) > 0 THEN
    RETURN TRUE.
  ELSE 
    RETURN FALSE.

END. /* FUNCTION IsOper () */

FUNCTION IsNumber RETURNS LOGICAL (INPUT cExpression AS CHARACTER):

  DEF VAR d AS DECIMAL NO-UNDO.

  ASSIGN d = DECIMAL (cExpression) NO-ERROR.
  IF ERROR-STATUS:ERROR THEN
    RETURN FALSE.
  ELSE 
    RETURN TRUE.

END. /* FUNCTION IsNumber () */

FUNCTION Plus RETURNS DECIMAL 
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a + b.

END. /* FUNCTION Plus */

FUNCTION Minus RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

/* MESSAGE "Made it into Plus().". */
  RETURN a - b.

END. /* FUNCTION Minus */

FUNCTION Divide RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a / b.

END. /* FUNCTION Divide */

FUNCTION Multiply RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a * b.

END. /* FUNCTION Multiply */

FUNCTION CalcRPN RETURNS DECIMAL (INPUT cExpression AS CHARACTER):

  DEF VAR dAccumulator AS DECIMAL NO-UNDO.
  DEF VAR dRegX        AS DECIMAL NO-UNDO.
  DEF VAR cToken       AS CHARACTER NO-UNDO.

  DEF VAR i AS INTEGER NO-UNDO.

  /* RPN always starts off with accumulator initialization */

  ASSIGN dAccumulator = DECIMAL (ENTRY(1, cExpression, " ")).

  /* Examine each token in the expression */

  DO i = 2 TO NUM-ENTRIES(cExpression, " "):

    /* Grab a symbol out of the expression. */

    ASSIGN cToken = ENTRY(i, cExpression, " ").
    ASSIGN cToken = TRIM(cToken).
    /* MESSAGE "cToken = " cToken. */

    /* If it is an operator, then perform the operation */
    /* and place the results back into the accumulator  */

    IF IsOper(cToken) THEN DO:

      /* MESSAGE "cToken is Oper " cToken. */
      IF cToken = "+" THEN ASSIGN dAccumulator = Plus(dAccumulator, dRegX).
      IF cToken = "-" THEN ASSIGN dAccumulator = Minus(dAccumulator, dRegX).
      IF cToken = "*" THEN ASSIGN dAccumulator = Multiply(dAccumulator, dRegX).
      IF cToken = "/" THEN ASSIGN dAccumulator = Divide(dAccumulator, dRegX).
      NEXT.

    END. /* IF IsOper(cToken) */

    /* If it is a number, throw it into a register and loop */
    /* for the expected operator.                           */

    IF IsNumber(cToken) THEN DO:

      ASSIGN dRegX = DECIMAL (cToken).
      /* MESSAGE "Assigned dRegX = " dRegX. */
      NEXT.

    END. /* IF IsNumber(cToken) */

    ELSE DO:
      /* MESSAGE "Cannot Recognize Token " cToken. */
    END.
  END. /* DO i = 1 TO NUM-ENTRIES(cExpression, " ") */

  RETURN dAccumulator.

END. /* FUNCTION CalcRPN */

