/*
 Copyright (c) 2002 Amduus Information Works, Inc.  
 scott_auge@yahoo.com sauge@amduus.com

 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation the
 rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 sell copies of the Software, and to permit persons to whom the Software
 is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
 KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
*/ 

FUNCTION AlgParm RETURNS CHARACTER
(INPUT cExpression AS CHARACTER,
 INPUT dValue      AS DECIMAL,
 INPUT cVarName    AS CHARACTER):

  RETURN REPLACE(cExpression, cVarName, STRING(dValue)).

END.


FUNCTION CleanExpr RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR cResult     AS CHARACTER NO-UNDO.

  /* All parenthesis need spaces around them */
  
  ASSIGN cResult = REPLACE(cExpression, "(", " ( ").
  ASSIGN cResult = REPLACE(cResult, ")", " ) ").
  
  /* All the +, * and / need spaces around them.  - is a seperate issue */
  /* so assume the user has clue about it's use as it means negative    */
  /* number also.                                                       */
  
  ASSIGN cResult = REPLACE(cResult, "+", " + ").
  ASSIGN cResult = REPLACE(cResult, "*", " * ").
  ASSIGN cResult = REPLACE(cResult, "/", " / ").
  
  /* All ] need spaces around them */
  
  ASSIGN cResult = REPLACE(cResult, "]", " ] ").
  
  /* All [ need a space behind them */
  
  ASSIGN cResult = REPLACE(cResult, "[", " [ ").
  
  /* The function beginning argument delimiter cannot have a space */
  /* in front of it.  In case there are multiple spaces in front,  */
  /* we loop until it is gone.                                     */
  
  REPEAT:
    IF INDEX(cResult, " [") <> 0 THEN
      ASSIGN cResult = REPLACE(cResult, " [", "[").
    ELSE
     LEAVE.
  END.
  
  /* Clear out double spaces and return the result */
  
  REPEAT:
    IF INDEX(cResult, "  ") <> 0 THEN
      ASSIGN cResult = REPLACE(cResult, "  ", " ").
    ELSE
     LEAVE.
  END.
 
  RETURN cResult.
 
END.

FUNCTION IsOper RETURNS LOGICAL (INPUT cExpression AS CHARACTER):

  IF INDEX ("+-*/", cExpression) > 0 THEN
    RETURN TRUE.
  ELSE 
    RETURN FALSE.

END. /* FUNCTION IsOper () */

FUNCTION IsNumber RETURNS LOGICAL (INPUT cExpression AS CHARACTER):

  DEF VAR d AS DECIMAL NO-UNDO.

  ASSIGN d = DECIMAL (cExpression) NO-ERROR.
  IF ERROR-STATUS:ERROR THEN
    RETURN FALSE.
  ELSE 
    RETURN TRUE.

END. /* FUNCTION IsNumber () */

FUNCTION Plus RETURNS DECIMAL 
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a + b.

END. /* FUNCTION Plus */

FUNCTION Minus RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

/* MESSAGE "Made it into Plus().". */
  RETURN a - b.

END. /* FUNCTION Minus */

FUNCTION Divide RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a / b.

END. /* FUNCTION Divide */

FUNCTION Multiply RETURNS DECIMAL
(INPUT a AS DECIMAL,
 INPUT b AS DECIMAL):

  RETURN a * b.

END. /* FUNCTION Multiply */

FUNCTION CalcRPN RETURNS DECIMAL (INPUT cExpression AS CHARACTER):

  DEF VAR dAccumulator AS DECIMAL NO-UNDO.
  DEF VAR dRegX        AS DECIMAL NO-UNDO.
  DEF VAR cToken       AS CHARACTER NO-UNDO.

  DEF VAR i AS INTEGER NO-UNDO.

  /* RPN always starts off with accumulator initialization */

  ASSIGN dAccumulator = DECIMAL (ENTRY(1, cExpression, " ")).

  /* Examine each token in the expression */

  DO i = 2 TO NUM-ENTRIES(cExpression, " "):

    /* Grab a symbol out of the expression. */

    ASSIGN cToken = ENTRY(i, cExpression, " ").
    ASSIGN cToken = TRIM(cToken).
    /* MESSAGE "cToken = " cToken. */

    /* If it is an operator, then perform the operation */
    /* and place the results back into the accumulator  */

    IF IsOper(cToken) THEN DO:

      /* MESSAGE "cToken is Oper " cToken. */
      IF cToken = "+" THEN ASSIGN dAccumulator = Plus(dAccumulator, dRegX).
      IF cToken = "-" THEN ASSIGN dAccumulator = Minus(dAccumulator, dRegX).
      IF cToken = "*" THEN ASSIGN dAccumulator = Multiply(dAccumulator, dRegX).
      IF cToken = "/" THEN ASSIGN dAccumulator = Divide(dAccumulator, dRegX).
      NEXT.

    END. /* IF IsOper(cToken) */

    /* If it is a number, throw it into a register and loop */
    /* for the expected operator.                           */

    IF IsNumber(cToken) THEN DO:

      ASSIGN dRegX = DECIMAL (cToken).
      /* MESSAGE "Assigned dRegX = " dRegX. */
      NEXT.

    END. /* IF IsNumber(cToken) */

    ELSE DO:
      /* MESSAGE "Cannot Recognize Token " cToken. */
    END.
  END. /* DO i = 1 TO NUM-ENTRIES(cExpression, " ") */

  RETURN dAccumulator.

END. /* FUNCTION CalcRPN */


/********************************************************************/
/* Define your function here                                        */
/********************************************************************/

/* Sample function with two arguments */

FUNCTION Power RETURNS CHARACTER (INPUT cArgs AS CHARACTER):

  DEF VAR cArg1 AS CHARACTER NO-UNDO.
  DEF VAR cArg2 AS CHARACTER NO-UNDO.

  DEF VAR dResult AS DECIMAL NO-UNDO.

  /* Args come in a single string delimited with , */

  ASSIGN cArg1 = ENTRY (1, cArgs)
         cArg2 = ENTRY (2, cArgs).

  /* Do your computation */

  ASSIGN dResult = EXP(DECIMAL(cArg1), DECIMAL(cArg2)).

  /* Return a string! */

  RETURN STRING (dResult).

END. /* FUNCTION Power */


/********************************************************************/
/* Add your function to be evaluated here.                          */
/********************************************************************/

FUNCTION FuncEval RETURNS CHARACTER (INPUT cExpression AS CHARACTER ):

  DEF VAR cSubExpression       AS CHARACTER NO-UNDO.
  DEF VAR iSubExpressionIndex  AS INTEGER NO-UNDO.
  DEF VAR cReplaceExpression   AS CHARACTER NO-UNDO.
  DEF VAR cArgExpression       AS CHARACTER NO-UNDO.
  DEF VAR cEvalExpression      AS CHARACTER NO-UNDO.

  /* We always evalute functions from the right to the left */
  /* to make sure they are evaluted according to algebraic  */
  /* rules.  We repeat to make sure we get em all!          */
  /* To add your own function:                              */
  /*   1) Create a function as above                        */
  /*   2) Determine the name of the function in the expres- */
  /*      sion                                              */
  /*   3) Add AlgFuncTemplate.i  First argument is the ex-  */
  /*      pression name, the second is the function name in */
  /*      in your code.                                     */

  /******************* Begin Example Template *******************/

  {AlgFuncTemplate.i Power Power}

  /******************* End Example Template *******************/

  RETURN cExpression.

END.

/* Take an algebraic expression and convert into RPN */
/* Needs to be included with CalcRPN.i               */

FUNCTION CalcAlg RETURNS CHARACTER (INPUT cExpression AS CHARACTER) FORWARD.

/* Locate first multiplication in expression, calc RPN, and replace */
/* the expression with the result.                                  */

FUNCTION SubMultiply RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR iWhereMult AS INTEGER NO-UNDO.
  DEF VAR i          AS INTEGER NO-UNDO.
  DEF VAR cMultExpr  AS CHARACTER NO-UNDO.
  DEF VAR cOpr1      AS CHARACTER NO-UNDO.
  DEF VAR cOpr2     AS CHARACTER NO-UNDO.

  /* IF there is no multiplier, then bail */

  IF INDEX (cExpression, "*") = 0 THEN RETURN cExpression.

  /* Locate a multiplier operator and it's operands               */
  /* This is a bad loop, should use index, but convenient to work */
  /* with ENTRY than INDEX.                                       */

  DO i = 1 TO NUM-ENTRIES (cExpression, " "):

     IF ENTRY(i, cExpression, " ") = "*" THEN DO:
       ASSIGN iWhereMult = i.
       LEAVE.
     END. /* IF ENTRY(i, cExpression, " ") = "*" */
       
  END. /* DO i = 1 TO NUM-ENTRIES (cExpression, " ") */

  /* Pull out our multiplication expression for substitution */

  ASSIGN
    cOpr1 = ENTRY(iWhereMult - 1, cExpression, " ")
    cOpr2 = ENTRY(iWhereMult + 1, cExpression, " ").

  /* Call out to RPNCalc to get a result to replace expression with */

  ASSIGN cMultExpr = STRING(CalcRPN(cOpr1 + " " + cOpr2 + " *")).

  /* Perform the substitution in the expression with the */
  /* calculated result.                                  */

  ASSIGN cExpression = REPLACE (cExpression, cOpr1 + " * " + cOpr2, cMultExpr).

  RETURN cExpression.

END. /* FUNCTION SubMultiply */

/* Locate first division in expression, calc RPN, and replace */
/* the expression with the result.                            */

FUNCTION SubDivision RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR iWhereDiv AS INTEGER NO-UNDO.
  DEF VAR i          AS INTEGER NO-UNDO.
  DEF VAR cDivExpr  AS CHARACTER NO-UNDO.
  DEF VAR cOpr1      AS CHARACTER NO-UNDO.
  DEF VAR cOpr2     AS CHARACTER NO-UNDO.

  /* IF there is no divisor, then bail */

  IF INDEX (cExpression, "/") = 0 THEN RETURN cExpression.

  /* Locate a divisor operator and it's operands                  */
  /* This is a bad loop, should use index, but convenient to work */
  /* with ENTRY than INDEX.                                       */

  DO i = 1 TO NUM-ENTRIES (cExpression, " "):

     IF ENTRY(i, cExpression, " ") = "/" THEN DO:
       ASSIGN iWhereDiv = i.
       LEAVE.
     END. /* IF ENTRY(i, cExpression, " ") = "/" */
       
  END. /* DO i = 1 TO NUM-ENTRIES (cExpression, " ") */

  /* Pull out our division expression for substitution */

  ASSIGN
    cOpr1 = ENTRY(iWhereDiv - 1, cExpression, " ")
    cOpr2 = ENTRY(iWhereDiv + 1, cExpression, " ").

  /* Call out to RPNCalc to get a result to replace expression with */

  ASSIGN cDivExpr = STRING(CalcRPN(cOpr1 + " " + cOpr2 + " /")).

  /* Perform the substitution in the expression with the */
  /* calculated result.                                  */

  ASSIGN cExpression = REPLACE (cExpression, cOpr1 + " / " + cOpr2, cDivExpr).

  RETURN cExpression.

END. /* FUNCTION SubDivision */

/* Function looks for groupings by () and evaluates the expression, */
/* replacing the () groupings with the evaluations.                 */
/* Note it looks for _( where _ is a space.  This way it won't mess */
/* with ( associated with functions which need the ( abutted to the */
/* function name as in "sin(" not "sin ("                           */
/* Hence, if you want groupings you must do this 3 + ( 9 - 6 ), NOT */
/* 3 +(9 - 6).                                                      */
/* Note, for functions, I am thinking of [ ] as argument delimiters */

FUNCTION GrpOrder RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  DEF VAR cSubExpression      AS CHARACTER NO-UNDO.
  DEF VAR iSubExpressionIndex AS INTEGER NO-UNDO.
  DEF VAR cEvalExpression     AS CHARACTER NO-UNDO.

  REPEAT:

    /* Nothing to group evaluate by. */

    IF INDEX(cExpression, " (") = 0 THEN RETURN TRIM(cExpression).
  
    /* Chop out part of the expression with _(_ in it       */
    /* to the next _)_ symbol.                              */
    /* We move right to left as the symbols tend to be      */
    /* present later in the string but need to be evaluated */
    /* before, if that makes any sense to the reader.       */
  
    ASSIGN cSubExpression = SUBSTRING(cExpression, R-INDEX(cExpression, " ( ")).
/* MESSAGE "OrderGrp:cSubExpression:1: " cSubExpression  . */
    /* Find the compliment _)_ to the _(_ */
  
    ASSIGN cSubExpression = SUBSTRING(cSubExpression, 1, INDEX(cSubExpression, " ) ")).
/* MESSAGE "OrderGrp:cSubExpression:2: " cSubExpression  . */
  
    /* Remove the _(_ and _)_ and send the sub-expression on it's */
    /* way to Alg2PRN for evaluation.                             */
  
    ASSIGN cSubExpression = REPLACE(cSubExpression, " ( ", "").
    ASSIGN cSubExpression = REPLACE(cSubExpression, " ) ", "").
    ASSIGN cSubExpression = TRIM(cSubExpression).
/* MESSAGE "OrderGrp:cSubExpression:3: " cSubExpression  . */
  
    ASSIGN cEvalExpression = CalcAlg(cSubExpression).
  
    /* Replace the sub-expression with the evaluated expression */

    ASSIGN cExpression = REPLACE(cExpression, "( " + cSubExpression + " )", cEvalExpression).

  END. /* REPEAT */

END. /* FUNCTION GrpOrder () */

/* Looks for a * or / in the expression and calls out to Subxxxx */
/* to perform the calculation and replacement.  Goes left to     */
/* right though the expression till nothing but + and - left.    */

FUNCTION OrderOpr RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  /* Determine if there are () grouping symbols and evaluate */
  /* accordingly.                                            */

  ASSIGN cExpression = GrpOrder(cExpression).

  /* Calculate functions here, they should reduce to numbers */
  /* for basic operators to work on.                         */

  ASSIGN cExpression = FuncEval (cExpression).

  REPEAT:

    /* Determine our stop condition */

    IF INDEX (cExpression, "*") = 0 AND INDEX (cExpression, "/") = 0 THEN RETURN cExpression.

    /* Determine which operation occurs first and call it's calculator */

    /* There is nothing but * in the expression */

    IF INDEX (cExpression, "/") = 0 THEN DO:
      ASSIGN cExpression =  SubMultiply(cExpression).
      NEXT.
    END.

    /* There is nothing but / in the expression */

    IF INDEX (cExpression, "*") = 0 THEN DO:
      ASSIGN cExpression =  SubDivision(cExpression).
      NEXT.
    END.

    /* We have * and / in the expression */

    IF INDEX (cExpression, "*") < INDEX (cExpression, "/") THEN 
        ASSIGN cExpression = SubMultiply(cExpression).
      ELSE 
        ASSIGN cExpression = SubDivision(cExpression).

  END. /* REPEAT */

END. /* FUNCTION OrderOpr */

FUNCTION Alg2RPN RETURNS CHARACTER (INPUT cExpression AS CHARACTER ):

  DEF VAR i            AS INTEGER NO-UNDO.
  DEF VAR cToken       AS CHARACTER NO-UNDO.
  DEF VAR cAccumulator AS CHARACTER NO-UNDO.
  DEF VAR cRegX        AS CHARACTER NO-UNDO.
  DEF VAR cRPN         AS CHARACTER NO-UNDO.

  /* Apply Order of Operations for Multiplication and division. */
  /* Next version will deal with () order of operations.         */

  ASSIGN cExpression = OrderOpr(cExpression).

  /* First Number in Algebra always Accumulator. */

  ASSIGN cRPN = ENTRY(1, cExpression, " ").

  DO i = 2 TO NUM-ENTRIES(cExpression, " "):

    ASSIGN cToken = ENTRY(i, cExpression, " ").

    IF IsOper(cToken) THEN DO:

      /* Remember the operator token */
      ASSIGN cRegX = cToken.

      ASSIGN i = i + 1.  /* Move to next operand */
      ASSIGN cToken = ENTRY(i, cExpression, " ").

      /* Attach Operand and operation into RPN string */
      ASSIGN cRPN = cRPN
                  + " "
                  + cToken
                  + " "
                  + cRegX. 
    END. /* IF IsOper(cToken) */

  END. /* DO */

  RETURN cRPN.

END. /* FUNCTION Alg2RPN */

/* Entry point for algebraic calculations */

FUNCTION CalcAlg RETURNS CHARACTER (INPUT cExpression AS CHARACTER):

  RETURN STRING(CalcRPN(Alg2RPN(CleanExpr(cExpression)))).

END.

/* Main routine! */

DEF INPUT PARAMETER t AS CHARACTER NO-UNDO.
DEF OUTPUT PARAMETER r AS CHARACTER NO-UNDO.

ASSIGN r = CalcAlg(t).

