
DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/Denkh/src/PDFSvr/RCS/PDFSvr.p,v 1.1 2002/07/12 03:41:08 sauge Exp sauge $" NO-UNDO.

/* The PDF Server is a process that runs in the background and performs   */
/* a continous loop looking for PDFReport records with a ReportStatus     */
/* of SUBMITTED.  Processing the report changes the status to PROCESSING. */
/* Once completed the status is changed to DONE and the client can use    */
/* PDFClt/PDFSvrGet.p to pull the PDF file from the system.               */

/* This program needs to run on a Linux machine with Ghostscript and      */
/* groff on it.  Such a server can be built for you by Amduus Information */
/* Works, Inc.                                                            */

/* Multiple instances of this program can run at the same time for load   */
/* bearing tweeking.                                                      */

/* Just kill the process to shutdown this processor.                      */
/* Newer versions will include a cleaner shutdown method.                 */

DEF VAR cPDFReportID     AS CHARACTER NO-UNDO.
DEF VAR cOSFileID        AS CHARACTER NO-UNDO.
DEF VAR cError           AS CHARACTER NO-UNDO.

OUTPUT TO /dev/tty UNBUFFERED.

REPEAT:

  FIND FIRST PDFReport EXCLUSIVE-LOCK
  WHERE PDFReport.ReportStatus = "SUBMITTED"
  NO-ERROR.

  /* Determine if the report record is available.  If not, then wait a bit */
  /* and try again.                                                        */

  IF NOT AVAILABLE PDFReport THEN DO:

    PAUSE 2 NO-MESSAGE.
    MESSAGE "Pausing".
    NEXT.

  END.  /* IF NOT AVAILABLE PDFReport */

  /* We got one!  Change the status so PDFSvrWatch.p can pick up what is */
  /* happening over here.  Release it so that the client can pick up on  */
  /* this at their side.                                                 */

  MESSAGE "Processing Report Request: " PDFReport.PDFReportID.

  ASSIGN PDFReport.ReportStatus = "PROCESSING".
  ASSIGN cPDFReportID = PDFReport.PDFReportID.

  RELEASE PDFReport.

  /* Refind the PDFFile with a NO-LOCK status and process it */

  FIND PDFReport NO-LOCK
  WHERE PDFReport.PDFReportID = cPDFReportID
  NO-ERROR.

  /* Pull out the groff stuff. File is named with the PDFReportID  */
  /* as since that is unique it should be unique on the file sytem */

  MESSAGE "Pulling out troff source: /tmp/" PDFReport.PDFReportID.

  RUN file/RetrieveFile.p 
  (INPUT PDFReport.INOSFileID,
   INPUT PDFReport.PDFReportID,
   INPUT "/tmp/",
   OUTPUT cError).

  /* Run it though the routines that will plop out a PDF file */

  MESSAGE "Creating PDF: " "/tmp/" + PDFReport.PDFReportID + ".pdf".

  RUN groff2pdf.p 
  (INPUT "/tmp/" + PDFReport.PDFReportID,
   INPUT "/tmp/" + PDFReport.PDFReportID + ".pdf").

  /* Read the PDF File into the DB */

  MESSAGE "Storing PDF: /tmp/" PDFReport.PDFReportID + ".pdf".

  RUN file/StoreFile.p 
  (INPUT PDFReport.PDFReportID + ".pdf",
   INPUT "/tmp/",
   INPUT "PDFReport",
   OUTPUT cOSFileID,
   OUTPUT cError).

  /* Update the PDFReport record to note it is processed and ready for the */
  /* client.                                                               */

  FIND PDFReport EXCLUSIVE-LOCK
  WHERE PDFReport.PDFReportID = cPDFReportID
  NO-ERROR.

  ASSIGN 
  PDFReport.ReportStatus = "DONE"
  PDFReport.OutOSFileID = cOSFileID.

  MESSAGE "Report " cPDFReportID " marked DONE".

END. /* REPEAT: */

