/* api_troffprs.p
 *
 * Convert an embedded troff file into a .p.  This can then be called via
 * the genpdf.p program to make the result set into a pdf.
 *
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2002 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/Denkh/src/RCS/api_troffprs.p,v 3.0 2003/06/25 07:14:01 sauge Exp $" NO-UNDO.

DEF INPUT PARAMETER InFile   AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER OutFile  AS CHARACTER NO-UNDO.

DEF VAR IsTROFF   AS LOGICAL NO-UNDO.
DEF VAR Line     AS CHARACTER NO-UNDO.
DEF VAR C        AS CHARACTER NO-UNDO.
DEF VAR i        AS INTEGER NO-UNDO.
DEF VAR T        AS CHARACTER NO-UNDO.
DEF VAR IsFirstTick AS LOGICAL NO-UNDO.

/* Determine what we are putting in, and what we are setting out. */

INPUT FROM VALUE(InFile).
OUTPUT TO VALUE(OutFile).

/* Put out the word this is a generated file */

PUT UNFORMATTED "/* Warning - this is a generated file from " InFile " */~n".
PUT UNFORMATTED "/* Generated by tools from Amduus Information Works, Inc. */~n".
PUT UNFORMATTED "/* http://www.amduus.com    Scott Auge sauge@amduus.com */~n".
PUT UNFORMATTED SKIP(2).

PUT UNFORMATTED "DEF INPUT PARAMETER cOutputFile AS CHARACTER NO-UNDO.~n".
PUT UNFORMATTED "DEF STREAM StdOut.~n".
PUT UNFORMATTED "OUTPUT STREAM StdOut TO VALUE(cOutputFile).~n".
PUT UNFORMATTED SKIP(1).

/* Read in lines one by one from the InFile.  If we encounter a <!--WSS, */
/* then we are no longer in TROFF, and so leave those alone.  We keep     */
/* track with the IsTROFF flag.  When we encounter the token, set the     */
/* flag, when we encounter the --> token AND IsTROFF, then we unset the   */
/* the flag and parse the goods for ` characters.                        */

ASSIGN IsTROFF = TRUE.

REPEAT:

  IMPORT UNFORMATTED Line.

  /* If we encounter the troff tag, comment and set we are in 4gl land */

  IF INDEX (Line, "<!--TS") <> 0
  OR INDEX (Line, "<script language=~"troffscript~">") <> 0 THEN DO:

    ASSIGN IsTROFF = FALSE.
    ASSIGN Line = REPLACE (Line, "<!--TS", "").
    ASSIGN Line = REPLACE (Line, "<script language=~"troffscript~">", "").

  END.

  /* If we are not in TROFF land, but in 4GL land, and encounter the troff */
  /* tag, then comment and replace. Set the flag we are in TROFF land.        */

  IF (INDEX (Line, "-->") <> 0 AND IsTROFF= FALSE)
  OR (INDEX (Line, "</script>") <> 0 AND IsTROFF = FALSE) THEN DO:


    ASSIGN Line = REPLACE (Line, "-->", "").
    ASSIGN Line = REPLACE (Line, "</script>", "").
    ASSIGN IsTROFF = TRUE.
    PUT UNFORMATTED Line "~n".
    NEXT.

  END.

  /* If we are in TROFF, there may be progress meta characters mixed in with */
  /* the javascript (things like { and } and ' which cause horrible prob-   */
  /* lems.  We want to ~ these in the TROFF, but we do NOT want to tilde the */
  /* the ones in `` phrases cuz that is 4GL and we assume the programmer    */
  /* has a clue about how to do such things.                                */

  IF IsTROFF THEN DO:

    ASSIGN 
    T = ""
    IsFirstTick = FALSE.

    DO i = 1 TO LENGTH (Line):

      ASSIGN c = SUBSTRING (Line, i, 1).

      /* Arrangement of these IFs is important, think about it */

      IF c = '`' AND IsFirstTick = TRUE THEN
        ASSIGN IsFirstTick = FALSE. /* Saying we are not in a `` phrase */
      ELSE 
      IF c = '`' AND IsFirstTick = FALSE THEN 
        ASSIGN IsFirstTick = TRUE.  /* Saying we are in a `` phrase */

      /* When in TROFF, but not in `` phrase, we want progress to take */
      /* these chars literally, not as meta chars meaning in the 4GL  */

      IF c = "~}" AND NOT IsFirstTick THEN ASSIGN c = "~~~}".
      IF c = "~{" AND NOT IsFirstTick THEN ASSIGN c = "~~~{".
      IF c = "~"" AND NOT IsFirstTick THEN ASSIGN c = "~~~"".
      IF c = "~'" AND NOT IsFirstTick THEN ASSIGN c = "~~~'".

      /* In javascript, one can \" to make the " literal to the javascript */
      /* interpreter.  We want to leave those alone!                       */

      IF c = "~\" AND NOT IsFirstTick THEN ASSIGN c = "~~~\". 

      ASSIGN T = T + c.

    END.  /* DO i = 1 TO LENGTH (Line) */

    ASSIGN 
    Line = T
    T = "".

  END. /* IF IsTROFF THEN */

  IF IsTROFF THEN DO:

    /* If we are in TROFF, and it is a blank line, then don't bother with it */
    /* all we do is spin useless code for not.                              */

    IF Line = "" THEN NEXT.

    /* If there are back ticks, we want to change those into appropriate */
    /* string operators.  First we handle the first back tick, then we   */
    /* handle the last back tick.  Note that both back ticks must be on  */
    /* same line!                                                        */

    ASSIGN IsFirstTick = TRUE.

    DO i = 1 TO LENGTH (Line):

      ASSIGN c = SUBSTRING (Line, i, 1).

      IF C <> "`" THEN DO:
        ASSIGN T = T + c.
        NEXT.
      END. /* IF C <> "`" */

      /* Here we hande C when it is a back tick */

      IF IsFirstTick THEN DO:

        ASSIGN T = T + "' ".
        ASSIGN IsFirstTick = FALSE.

      END. /* IF IsFirstTick */

      ELSE DO:

        ASSIGN T = T + " '".
        ASSIGN IsFirstTick = TRUE.

      END. /* IF NOT IsFirstTick */

    END.  /* DO i = 1 TO LENGTH(Line) */

    ASSIGN line = T
           T = "".

    /* Spit out the line */

    IF IsTROFF THEN PUT UNFORMATTED "~nPUT STREAM StdOut UNFORMATTED '".
    PUT UNFORMATTED Line .
    IF IsTROFF THEN PUT UNFORMATTED "~~n'.".

  END.

  ELSE 
    PUT UNFORMATTED Line "~n".

END. /* REPEAT */

PUT UNFORMATTED SKIP(2).
PUT UNFORMATTED "OUTPUT STREAM StdOut CLOSE.~n~n".

INPUT CLOSE.
OUTPUT CLOSE.


