
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/*
 * WriteState.p
 * Written by Scott Auge
 *
 * Given a session id, category, and state name, set the value.
 *
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/Denkh/src/state/RCS/WriteState.p,v 1.1 2003/07/11 04:23:48 sauge Exp sauge $" NO-UNDO.

DEF INPUT PARAMETER pSessionID   AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pCategory    AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pName        AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pValue       AS CHARACTER NO-UNDO.

DEF VAR pError AS CHARACTER NO-UNDO.

/* Determine if there is a state of this session.  If so, we will just over- 
 * write the values.  If there is not, we will create a record.  Note we look
 * up the record with a NO-WAIT option in case it is used by some other
 * program.  It is up to the calling program to determine what it should do
 * under this condition.  (Recommended is to pause and try again.)
 */

FIND WebState EXCLUSIVE-LOCK
WHERE WebState.SessionID = pSessionID
  AND WebState.Category = pCategory
  AND WebState.Name = pName
  NO-WAIT NO-ERROR.

IF NOT AVAILABLE WebState THEN
  IF LOCKED WebState THEN DO:
    ASSIGN 
    pError = "LOCKED".
    RETURN.
  END. /* NOT AVAILABLE AND LOCKED! */
  ELSE DO:
    CREATE WebState.
    ASSIGN
    WebState.SessionID = pSessionID
    WebState.Category = pCategory
    WebState.Name = pName.
  END.

ASSIGN
WebState.Data = pValue.

ASSIGN
pError = "NO_ERROR".
